
% this script trains the assessor network


% initialize random number generator
RandStream.setDefaultStream ...
     (RandStream('mt19937ar','seed',sum(100*clock)));

format compact
clear W1vect W2vect b1vect b2vect

% this is where W, b, eta are read in. These are the parameters of the autoassociator.
load ../autoassociator/autoparams

% get matrix showing features of consonants
load featmat featmat
% get list of triliteral roots not of form XYY
load ../buckwalter_data/buckwalter buckwalter_roots_nongem
wordmat=buckwalter_roots_nongem;
% wordmat- list of attested words in terms of triples of consonants
% featmat - featural description of each segment

% number of features in feat spec of a consonant
numfeat=size(featmat,1);
% length of input to assessor
inputlength=3*numfeat;
% number of words in roots list
numinputwords=size(wordmat,1);

% important parameter (number of learning epochs)
numepochs=1e5;   % should be 1e7 to reproduce results in paper
% important parameter (number of nodes in hidden layer)
hiddenlength=5;

% important parameter (damping rate of weights)
alpha=.00001;

% length of output layer  (assessment of a root)
outputlength=1;

% magnitude of initial random weights on connections
beta=0.1;

% small random initial weights for connections
W1=beta*randn(hiddenlength,inputlength);
b1=beta*randn(hiddenlength,1);
W2=beta*randn(outputlength,hiddenlength);
b2=beta*randn(outputlength,1);

% learning parameter delta changes with time
deltainitial=1;
deltafinal=.1;
deltaratio=(deltafinal/deltainitial)^(1/numepochs);
deltavect=deltainitial*deltaratio.^(1:numepochs);

% start epoch
for j=1:numepochs

  % occasionally display the epoch number and store information
  if mod(j,1e4)==0
    j
    W1vect{j/1e4}=W1;
    W2vect{j/1e4}=W2;
    b1vect{j/1e4}=b1;
    b2vect{j/1e4}=b2;
  end
  
  % choose learning rate for this epoch
  delta=deltavect(j);

  % generate word through autoassociator
  choice=ceil(rand*numinputwords);
  pre_input_segs=wordmat(choice,:);
  input_segs=autoassoc_map(pre_input_segs,W,b,eta);
  input=[ featmat(:,input_segs(1))' featmat(:,input_segs(2))' featmat(:,input_segs(3))' ]';

  % real word or fake word?
  if (sum(abs(pre_input_segs-input_segs))==0)
    desired=1;
  else % fake word
   desired=-1;
  end
      
  % compute output of network
  hidden= sigmoid(W1*input + b1);
  output=sigmoid(W2*hidden + b2); 
  erro2=desired-output;

  % Begin Backpropagation
  dW2=delta*(erro2.*(1-output.^2)/2)*hidden';
  db2=delta*erro2.*(1-output.^2)/2;

  dh=W2'*(erro2.*(1-output.^2)/2);

  dW1=delta*(dh.*(1-hidden.^2)/2)*input';
  db1=delta*dh.*(1-hidden.^2)/2;

  W2=W2+dW2;
  b2=b2+db2;
  W1=W1+dW1;
  b1=b1+db1;

  % damping of amplitudes
  W1=W1*(1-delta*alpha);
  W2=W2*(1-delta*alpha);
  b1=b1*(1-delta*alpha);
  b2=b2*(1-delta*alpha);

  % end backpropagation
 
end

save assessor_params W1 W2 b1 b2

