
% this code simulates Experiment 1 of F & Z

% get assessor parameters
load ../assessor/assessor_params

% read in list of words in C1 C2 C3 form
fid=fopen('set_1_words');
set1words=fscanf(fid,'%i',[3 inf])';
fclose(fid);

% read in list of average acceptibility ratings
fid=fopen('set_1_ratings');
set1ratings=fscanf(fid,'%f',[1 inf])';
fclose(fid);

% how many words in set
numwordsexp=size(set1words,1);

% find experimental words on list of all words
load words legword
exp1_indices=zeros(numwordsexp,1);
for j=1:numwordsexp
  wo=set1words(j,:);
  exp1_indices(j)=find( (legword(:,1)==wo(1)) & (legword(:,2)==wo(2)) & (legword(:,3)==wo(3)) );
end

% From words.mat get precomputed monogram neighbours ocpvect for all possible
% words
load words monogram neighbours ocpvect allwords

% exp1_indices is the indices of the exp1 words in the master list.
% now get words in terms of natural classes
exp1_words=allwords(exp1_indices,:);

% apply net to non-words.  extract monogram,neighbours,ocpvect for non-words.
y=applynet(exp1_words,W1,W2,b1,b2);
x1=monogram(exp1_indices);
x2=neighbours(exp1_indices);
x3=ocpvect(exp1_indices);

figure(1)

clf

subplot(2,2,1),hist(x1),title('monogram')
subplot(2,2,2),hist(x2),title('neighbours')
subplot(2,2,3),hist(x3),title('ocp viol')
subplot(2,2,4),hist(y),title('network output')

meany=mean(y);
SStot=sum((y-meany).^2);

% do over all possible subsets of three variables
X=[x1 x2 x3];
index{1}=[];
index{2}=[1];
index{3}=[2];
index{4}=[3];
index{5}=[ 1 2];
index{6}=[ 2 3];
index{7}=[ 1 3];
index{8}=[ 1 2 3];

for j=1:8;
  A=[ones(size(x1)) X(:,index{j})];
  c=A\y;

  SSreg(j)=sum((A*c-meany).^2);
  Rsqvect(j)=SSreg(j)/SStot;

end

out=Rsqvect

disp('Fraction of variation explained by...')
disp('monogram probability')
out(2)
disp('number of neighbours')
out(3)
disp('OCP violations')
out(4)


figure(2) 
clf
averboxnoviol=zeros(2);
averboxocpviol=zeros(2);
averbox=zeros(2,2,2);
cut1=4e-5;
% low prob, low neighbours, no ocp viol
averboxnoviol(1,1)=mean(y(find( x1<cut1 & x2<=10 & x3==0)));
% high prob, low neighbours, no ocp viol
averboxnoviol(2,1)=mean(y(find( x1>=cut1 & x2<=10 & x3==0)));
% low prob, high neighbours, no ocp viol
averboxnoviol(1,2)=mean(y(find( x1<cut1 & x2>11 & x3==0)));
% high prob, high neighbours, no ocp viol
averboxnoviol(2,2)=mean(y(find( x1>=cut1 & x2>11 & x3==0)));

averboxocpviol(1,1)=mean(y(find( x1<cut1 & x2<=10 & x3>0)));
averboxocpviol(2,1)=mean(y(find( x1>=cut1 & x2<=10 & x3>0)));
averboxocpviol(1,2)=mean(y(find( x1<cut1 & x2>11 & x3>0)));
averboxocpviol(2,2)=mean(y(find( x1>=cut1 & x2>11 & x3>0)));

plot([averboxnoviol(1,1:2) averboxnoviol(2,1:2)],'.')
hold on
plot([averboxocpviol(1,1:2) averboxocpviol(2,1:2)],'.r')
hold off

% do properanova
boxsums= [ sum(y(find( x1<cut1 & x2<=10 & x3==0))) ; ...
           sum(y(find( x1>=cut1 & x2<=10 & x3==0))) ; ...
           sum(y(find( x1<cut1 & x2>10 & x3==0))) ; ...
           sum(y(find( x1>=cut1 & x2>10 & x3==0))) ; ...
           sum(y(find( x1<cut1 & x2<=10 & x3>0))) ; ...
           sum(y(find( x1>=cut1 & x2<=10 & x3>0))) ; ...
           sum(y(find( x1<cut1 & x2>10 & x3>0))) ; ...
           sum(y(find( x1>=cut1 & x2>10 & x3>0)))  ];

F=anova23(boxsums,y)

figure(3) 
plot(set1ratings,y,'.');

cc=corrcoef([set1ratings y]);
disp('correlation between network and experiment')
disp(cc(1,2))


